;Mass storage device driver routines

 include "settings.inc"
  include "../ti83plus.inc"
 include "equates.inc"
 SEGMENT Main
 GLOBALS ON

 EXTERN IsDeviceConnected,USBErrorNoDevice,USBHostInit,USBErrorInit,USBErrorCont,DOS_fileRead,DOS_fileWrite,DOS_openFile
 EXTERN FAT_findFileInCluster,DOS_createFile,FAT_nameConvertTo11,FAT_getStartingCluster,DOS_fileSeek,FindPipe,AutoSetupNoInit
 EXTERN MSDCallBack,USBErrorAutoSetup,USBErrorClass,GetClass,DOS_openFileSC,strlen,sendReset,IGetKey,AutoSetup,DOS_getNextFile
 EXTERN cmpstrb,DOS_openFile_Cont,DispHexHL

getMSDData:
	;Gets address of HL offset in HL
	;Destroys nothing
	push de
	push hl
	ld hl,(MemPtr)
	ld de,offMSDMemPtr
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	pop hl
	add hl,de
	pop de
	ret

getMSDPtr:
	;Gets offMSDMemPtr into IX
	;Destroys nothing
 	push de
	push hl
	ld hl,(MemPtr)
	ld de,offMSDMemPtr
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	push de
	pop ix
	pop hl
	pop de
	ret

getSectorBuffer:
	;Gets offMSDSectorBuffer into DE
	;Destroys nothing
	push hl
	ld hl,(MemPtr)
	ld de,offMSDSectorBuffer
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	pop hl
	ret

MSDVersion:
	;Returns version number of mass storage driver and size of memory buffers required
	;Output:     B is major version
	;            C is minor version
	;            HL is size of required memory buffer (254 bytes)
	;            DE is size of required sector buffer (512 bytes)
	;            A is size of file handle (27 bytes)
	ld bc,MVersion
	ld hl,msdBufferSize
	ld de,msdSectorSize
	ld a,fHandleLen
	ret

MSD_Initialize:
	;Initializes a mass storage device using Bulk-Only Transport (and maybe Control-Bulk-Interrupt)
	;Input:	HL points to ~254-byte buffer for internal use
	;		DE points to sector buffer (almost always 512 bytes)
	;C set if failure
       LOG MSDInit
	push hl
	ld hl,(MemPtr)
	ld bc,offMSDSectorBuffer
	add hl,bc
	ld (hl),e
	inc hl
	ld (hl),d
       pop hl
       push hl
       ld bc,msdBufferSize
       B_CALL MemClear
	pop de
	ld hl,(MemPtr)
	ld bc,offMSDMemPtr
	add hl,bc
	ld (hl),e
	inc hl
	ld (hl),d
	call IsDeviceConnected
	jr z,USBErrorNoDevice
	ld hl,MSDDescBuf
	call getMSDData
	push hl
;	call USBHostInit
       ld de,MSDCallBack
       call AutoSetup
	pop hl
	jr c,USBErrorInit
	push hl
	call GetClass
	pop hl
	cp 8
	jr nz,USBErrorClass
	ld a,b
       cp 4
       jr z,$F
	cp 6
	jr nz,USBErrorClass
$$:    push hl
       ld hl,(MemPtr)
       ld de,offFlags
       add hl,de
       res flagMSDCBI,(hl)
       res flagMSDInt,(hl)
       ex de,hl
       pop hl
       ;DE points to flag byte
	ld a,c
	cp 50h
	jr z,$F
       ex de,hl
       set flagMSDCBI,(hl)
       ex de,hl
       cp 1
       jr z,$F
       or a
       jr nz,USBErrorClass
       ex de,hl
       set flagMSDInt,(hl)
       ex de,hl
$$:	;ld de,MSDCallBack
;	call AutoSetupNoInit
;	jp c,USBErrorAutoSetup
	call getMSDPtr
	ld hl,MSDDescBuf
	call getMSDData
       ld bc,128
       B_CALL MemClear
       ld b,pipeInt
       ld c,pipeIn
       ld hl,MSDDescBuf
       call getMSDData
       call FindPipe
       jr c,$F
       ld hl,(MemPtr)
       push hl
       ld de,offFlags
       add hl,de
;       set flagMSDCBI,(hl) ;why? A BOT drive shouldn't have 3 endpoints...
       set flagMSDInt,(hl) ;I think a third endpoint on a BOT drive is worthless to us anyway
       pop hl
       ld de,offMSDInterruptPipe
       add hl,de
       ld (hl),b
$$:    ld b,pipeBulk
       ld c,pipeOut
	ld hl,MSDDescBuf
	call getMSDData
       call FindPipe
	call getMSDPtr
	ld (ix+MSDOutPipe),b
	ld hl,MSDDescBuf
	call getMSDData
       ld bc,128
	push hl
       B_CALL MemClear
	pop hl
       ld b,pipeBulk
       ld c,pipeIn
 	call FindPipe
	call getMSDPtr
	ld (ix+MSDInPipe),b
;	ld (ix+MSDBOTState),0 MemClear above should take care of this
       ret

MSD_ImportVariable:
	;Imports *.8x* variable from connected USB flash drive and specified directory and stores in RAM/Flash
	;Input:	HL is ASCIIZ string of *.8x* file (path included)
	;		B is flags byte
	;			Bit 0: set if existing variable should be deleted in case duplicate is found
       ;                    Bit 1: set if variable should be archived
	;Output:	Variable is created in RAM or Flash
	;Returns C if problems
	;If duplicate was found and flag bit 0 is reset, C is returned
       ;If there is not enough free archive memory and flag bit 1 is set, C is returned
       ld a,b
	ld (flag_byte),a
	ld de,temp_handle
	call DOS_openFile
	ret c
	ld hl,temp_handle
	ld de,OP1
	ld bc,6
	call DOS_fileRead
	ret c
	ld hl,OP1+1
	ld de,bFileHeader+1
	ld b,5
$$:    ld a,(de)
	cp (hl)
	ld a,errMSDCorruptFile
	jr nz,USBErrorCont
	inc hl
	inc de
	djnz $B
	ld bc,temp_handle
	ld de,0
	ld hl,37h
	call DOS_fileSeek
	ld hl,temp_handle
	ld de,packet_length
	ld bc,2
	call DOS_fileRead
	ld bc,temp_handle
	ld de,0
	ld hl,3Bh
	call DOS_fileSeek
	B_CALL ZeroOP1
	ld hl,temp_handle
	ld de,OP1
	ld bc,9
	call DOS_fileRead
	ld bc,temp_handle
	ld de,39h
	ld hl,(packet_length)
	add hl,de
	ld de,0
	call DOS_fileSeek ;now pointing at size bytes
	ld hl,OP1 ;back up variable name
	ld de,temp_file
	ld bc,11
	ldir
	B_CALL ChkFindSym
	jr c,$F
	ld a,(flag_byte)
	and 00000001b ;overwrite duplicate variable?
	ld a,errMSDDuplicateFound
	jr z,USBErrorCont
	B_CALL DelVarArc
$$:    ld hl,temp_handle
	ld de,packet_length
	ld bc,2
	call DOS_fileRead
	ld hl,temp_file
	rst 20h
	;check for enough free memory
	ld hl,OP1+1
	call strlen
	ld d,0
	ld e,b
	ld hl,(packet_length)
	add hl,de
	ld de,9
	add hl,de ;HL is now how many bytes need to be free (including VAT entry)
	B_CALL EnoughMem
	ld a,errMSDNotEnoughMem
	jr c,USBErrorCont
	ld hl,(packet_length)
	ld a,(OP1)
	or a
	jr z,MSD_IVReal
	cp 01h
	jr z,MSD_IVRList
	cp 02h
	jr z,MSD_IVMatrix
	cp 03h
	jr z,MSD_IVYEqu
	cp 04h
	jr z,MSD_IVString
	cp 05h
	jr z,MSD_IVProg
	cp 06h
	jr z,MSD_IVProtProg
	cp 07h
	jr z,MSD_IVPict
	cp 08h
	jr z,MSD_IVGDB
	cp 0Ch
	jr z,MSD_IVCplx
	cp 0Dh
	jr z,MSD_IVCList
	cp 15h
	jr z,MSD_IVAppVar
       ld a,errMSDCorruptFile
	jr USBErrorCont
;HL/(packet_length) contain size bytes (this is row/col for matrices, etc.)
;OP1/temp_file contains variable type and name
;File seek pointer is just beyond size bytes
;This code might be cleaned up with B_CALL DataSize
MSD_IVReal:
	B_CALL CreateReal
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
	jr MSDIV_Continue
MSD_IVRList:
	dec hl
	dec hl
	B_CALL SetXXXXOP2
	B_CALL OP2ToOP1
	ld a,9
	B_CALL SetXXOP2
	B_CALL FPDiv
	B_CALL ConvOP1
	push de
	ld hl,temp_file
	rst 20h
	pop hl
	B_CALL CreateRList
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVMatrix:
	ld hl,temp_handle
	ld de,data_size
	ld bc,2
	call DOS_fileRead
	ld hl,(data_size)
	B_CALL CreateRMat
	ld bc,(packet_length)
	dec bc
	dec bc
	inc de
	inc de
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVYEqu:
	B_CALL CreateEqu
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVString:
	B_CALL CreateStrng
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVProg:
	B_CALL CreateProg
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVProtProg:
	B_CALL CreateProtProg
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVPict:
	B_CALL CreatePict
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVGDB:
	B_CALL CreatePict ;there is no CreateGDB function, manually changing VAT type byte
	ld (hl),08h
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVCplx:
	B_CALL CreateCplx
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVCList:
	dec hl
	dec hl
	B_CALL SetXXXXOP2
	B_CALL OP2ToOP1
	ld a,18
	B_CALL SetXXOP2
	B_CALL FPDiv
	B_CALL ConvOP1
	push de
	ld hl,temp_file
	rst 20h
	pop hl
	B_CALL CreateCList
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
       jr MSDIV_Continue
MSD_IVAppVar:
	B_CALL CreateAppVar
	ld bc,(packet_length)
	ld hl,temp_handle
	call DOS_fileRead
MSDIV_Continue:
       ld a,(flag_byte)
       and 00000010b
       ret z
       ;This actually might not work, I'm not sure (never been low on archive memory)
       in a,(6)
       ld (temp_mem),a
       ld hl,MSDIV_Error
       ld de,temp_mem+1
       push de
       ld bc,MSDIV_ErrorEnd-MSDIV_Error
       ldir
       pop hl
       call APP_PUSH_ERRORH
       ld hl,temp_file
       rst 20h
       B_CALL ChkFindSym
       B_CALL Arc_Unarc
       call APP_POP_ERRORH
       or a
       ret
MSDIV_Error:
       ld a,(temp_mem)
       out (6),a
       ld a,errMSDNotEnoughMem
       jp USBErrorCont
MSDIV_ErrorEnd:

MSD_ExportVariable:
	;Exports variable from RAM/Flash to connected USB flash drive and specified directory
	;Input:	(OP1) is type of variable to export
	;		OP1+1 contains ASCIIZ string of variable to export (similar to OP1 input for _ChkFindSym, etc.)
	;		HL points to ASCIIZ string of directory path to export variable ("/",0 is root)
	;Output:	*.8x* file of proper name and type is created in specified directory
       ;Notes:       Part of this might be cleaned up with B_CALL DataSize
	;Returns C if problems
	;Destroys OP1
	call FAT_getStartingCluster
	push hl
	ld hl,OP1
	ld de,OP1_temp
	ld bc,11
	ldir
	call MSDEV_fixName
	ld hl,OP1+1
	ld de,temp_file
$$:    ld a,(hl)
	or a
	jr z,$F
	ld (de),a
	inc hl
	inc de
	jr $B
$$:    ld a,'.'
	ld (de),a
	inc de
	ld a,'8'
	ld (de),a
	inc de
	ld a,'X'
	ld (de),a
	inc de
	ld hl,bFileTypes
	ld b,0
	ld a,(OP1)
	ld c,a
	add hl,bc
	ld a,(hl)
	ld (de),a
	inc de
	xor a
	ld (de),a
	ld hl,OP1_temp
	rst 20h
	ld hl,temp_file
	ld de,temp_mem
	call FAT_nameConvertTo11
	pop hl
	push hl
	ld de,temp_mem
	call FAT_findFileInCluster
	pop bc
	ld a,errMSDAlreadyExists
	jr nc,USBErrorCont
	push bc
	ld hl,temp_file
	call DOS_createFile
	pop bc
	ret c
	ld hl,temp_file
	ld de,temp_handle
	call DOS_openFileSC ;overloaded DOS_openFile, takes filename and starting cluster instead of full path
	ret c
	ld hl,temp_mem
	ld bc,53
	push bc
	push hl
	B_CALL MemClear
	ld hl,bFileHeader
	ld de,temp_mem
	ld bc,10
	ldir
	pop de
	pop bc
	ld hl,temp_handle
	call DOS_fileWrite
	ld hl,OP1_temp
	rst 20h
	B_CALL ChkFindSym
	ret c
	ld (address),de
	ld a,b
	ld (page),a
	or a
	jr z,MSDEV_AtData
	ld b,10
$$:    call MSDEV_getVarByte
	djnz $B
	ld b,a
	or a
	jr z,MSDEV_AtData ;just in case
$$:    call MSDEV_getVarByte
	djnz $B
MSDEV_AtData:
	ld a,(OP1_temp)
	or a
	jr z,MSDEV_IsReal
	cp 0Ch
	jr z,MSDEV_IsCplx
	jr MSDEV_AtSize
MSDEV_IsReal:
	ld hl,9
	jr $F
MSDEV_IsCplx:
	ld hl,18
	jr $F
MSDEV_AtSize:
	call MSDEV_getVarByte
	ld l,a
	push hl
	call MSDEV_getVarByte
	pop hl
	ld h,a
	or l
       ld a,errMSDVarEmpty
       jr z,USBErrorCont
	ld a,(OP1_temp)
	cp 02h
	jr z,MSDEV_IsMatrix
	cp 0Dh
	jr z,MSDEV_IsCList
	cp 01h
	jr nz,MSDEV_IsNotRList
	ld (data_size),hl
	push hl
	pop de
	add hl,hl ;*2
	add hl,hl ;*4
	add hl,hl ;*8
	add hl,de ;*9
	jr MSDEV_IsNotRList
MSDEV_IsMatrix:
	ld (data_size),hl
	B_CALL HTimesL
	push hl
	pop de
	add hl,hl ;*2
	add hl,hl ;*4
	add hl,hl ;*8
	add hl,de ;*9
	jr MSDEV_IsNotRList
MSDEV_IsCList:
	ld (data_size),hl
	push hl
	pop de
	add hl,hl ;*2
	add hl,hl ;*4
	add hl,hl ;*8
	add hl,de ;*9
	add hl,hl ;*18
MSDEV_IsNotRList:
	inc hl
	inc hl
;HL is now variable data size
$$:    push hl
	ld de,17
	add hl,de
	ld (temp_mem),hl
	ld hl,temp_mem+2
	ld a,0Dh
	call MSDEV_updateChecksum_R
	ld (hl),a
	inc hl
	ld (hl),0
	inc hl
	pop de
	ld a,e
	call MSDEV_updateChecksum
	ld (hl),e
	inc hl
	ld a,d
	call MSDEV_updateChecksum
	ld (hl),d
	inc hl
	push de
	ld a,(OP1_temp)
	call MSDEV_updateChecksum
	ld (hl),a
	inc hl
	ld de,OP1_temp+1
	ld b,8
$$:    ld a,(de)
	call MSDEV_updateChecksum
	ld (hl),a
	inc hl
	inc de
	dec b
	jr z,MSDEV_PastName
	ld a,(de)
	or a
	jr nz,$B
	ld a,b
	or a
	jr z,MSDEV_PastName
$$:    ld (hl),0
	inc hl
	djnz $B
MSDEV_PastName:
	ld (hl),0
	inc hl
	ld (hl),0
	inc hl
	pop de
	ld a,e
	call MSDEV_updateChecksum
	ld (hl),e
	inc hl
	ld a,d
	call MSDEV_updateChecksum
	ld (hl),d
	ld a,(OP1_temp)
	cp 0Dh
	jr z,MSDEV_IsCList2
	cp 02h
	jr z,MSDEV_IsCList2
	cp 01h
	jr nz,$F
MSDEV_IsCList2:
	dec de
	dec de
	ld (OP1),de
	ld (OP1+9),de
	ld hl,temp_handle
	ld de,temp_mem
	ld bc,19
	call DOS_fileWrite
	ld hl,temp_handle
	ld de,data_size
	ld a,(de)
	call MSDEV_updateChecksum
	inc de
	ld a,(de)
	call MSDEV_updateChecksum
	dec de
	ld bc,2
	call DOS_fileWrite
	jr MSDEV_WriteData
$$:    dec de
	dec de
	ld (OP1),de
	ld (OP1+9),de
	ld hl,temp_handle
	ld de,temp_mem
	ld bc,19
	call DOS_fileWrite
	ld a,(OP1_temp)
	or a
	jr z,MSDEV_WriteData
	cp 0Ch
	jr z,MSDEV_WriteData
	ld hl,temp_handle
	ld de,OP1+9
	ld a,(de)
	call MSDEV_updateChecksum
	inc de
	ld a,(de)
	dec de
	call MSDEV_updateChecksum
	ld bc,2
	call DOS_fileWrite
MSDEV_WriteData:
	ld b,128
	ld de,temp_mem
	ld hl,0
	ld (OP1+7),hl
$$:    call MSDEV_getVarByte
	call MSDEV_updateChecksum
	ld (de),a
	inc de
	ld hl,(OP1)
	dec hl
	ld (OP1),hl
	push hl
	ld hl,(OP1+7)
	inc hl
	ld (OP1+7),hl
	pop hl
	ld a,h
	or l
	jr z,$F
	djnz $B
$$:    ld hl,temp_handle
	ld de,temp_mem
	ld bc,(OP1+7)
	call DOS_fileWrite
	ld hl,(OP1)
	ld a,h
	or l
	jr nz,MSDEV_WriteData
	ld hl,temp_handle
	ld de,checksum
	ld bc,2
	jr DOS_fileWrite
MSDEV_updateChecksum_R:
	ld bc,0
	ld (checksum),bc
MSDEV_updateChecksum:
	push hl
	push de
	ld hl,(checksum)
	ld d,0
	ld e,a
	add hl,de
	ld (checksum),hl
	pop de
	pop hl
	ret
MSDEV_getVarByte:
	ld a,(page)
	or a
	jr nz,$F
	push de
	ld de,(address)
	ld a,(de)
	inc de
	ld (address),de
	pop de
	ret
$$:    push de
	push hl
	push bc
	ld b,a
	ld hl,(address)
	B_CALL LoadCIndPaged
	inc hl
	bit 7,h
	jr z,$F
	inc b
	res 7,h
	set 6,h
$$:	ld (address),hl
	ld a,b
	ld (page),a
	ld a,c
	pop bc
	pop hl
	pop de
	ret
MSDEV_fixName:
       ;Fix variable names
       ;Input:       OP1 is old name
       ;Output:      OP1 is new name
	ld a,(OP1)
	cp 01h
	jr z,MSDEV_fList
	cp 0Dh
	jr z,MSDEV_fList
	cp 02h
	jr z,MSDEV_fMatrix
	cp 03h
	jr z,MSDEV_fEquation
	cp 04h
	jr z,MSDEV_fString
	cp 07h
	jr z,MSDEV_fPicture
	cp 08h
	jr z,MSDEV_fGDB
	ret
MSDEV_fList:
	ld a,(OP1+2)
	cp 10
	jp p,$F
	add a,31h
	ld (OP1+2),a
$$:    ld a,'L'
	ld (OP1+1),a
	ret
MSDEV_fMatrix:
	ld a,(OP1+2)
	add a,'A'
	ld (OP1+1),a
	xor a
	ld (OP1+2),a
	ret
MSDEV_fPicture:
	ld a,(OP1+2)
	cp 9
	jr nz,$F
	sub 10
$$:    add a,31h
	ld (OP1+4),a
	xor a
	ld (OP1+5),a
	ld hl,sPic
	ld de,OP1+1
	ldi
	ldi
	ldi
	ret
sPic:
	DB "Pic"
MSDEV_fEquation:
	ld a,(OP1+2)
	cp 7Fh
	jp p,MSDEV_fe_s
	cp 3Fh
	jp p,MSDEV_fe_r
	cp 1Fh
	jp p,MSDEV_fe_l
	sub 10h
	cp 9
	jr nz,$F
	sub 10
$$:    add a,31h
	ld (OP1+2),a
	xor a
	ld (OP1+3),a
	ld a,'Y'
	ld (OP1+1),a
	ret
MSDEV_fe_s:
	sub 80h
	add a,75h
	ld (OP1+1),a
	xor a
	ld (OP1+2),a
	ret
MSDEV_fe_r:
	sub 40h
	add a,31h
	ld (OP1+2),a
	ld a,'r'
	ld (OP1+1),a
	xor a
	ld (OP1+3),a
	ret
MSDEV_fe_l:
	sub 20h
	push af
	and 1
	add a,'X'
	ld (OP1+1),a
	pop af
	sra a
	add a,31h
	ld (OP1+2),a
	ld a,'T'
	ld (OP1+3),a
	xor a
	ld (OP1+4),a
	ret
MSDEV_fString:
	ld a,(OP1+2)
	cp 9
	jr nz,$F
	sub 10
$$:    add a,31h
	ld (OP1+4),a
	xor a
	ld (OP1+5),a
	ld hl,sStr
	ld de,OP1+1
	ldi
	ldi
	ldi
	ret
sStr:
	DB "Str"
MSDEV_fGDB:
	ld a,(OP1+2)
	cp 9
	jr nz,$F
	sub 10
$$:    add a,31h
	ld (OP1+4),a
	xor a
	ld (OP1+5),a
	ld hl,sGDB
	ld de,OP1+1
	ldi
	ldi
	ldi
	ret
sGDB:
	DB "GDB"
bFileTypes:
	DB "NLMYSPPID",0,0,0,"CL",0,0,0,0,0,0,0,"V"
bFileHeader:
	DB "**TI83F*",01Ah,0Ah

msdDetect:
       ;Detects 83P/8XP files in specified directory with specified detection string
       ;Input:       HL is starting cluster of directory with level(s)
       ;             DE points to zero-terminated detection string
       ;             BC is entry number (pass -1 on first call to this function)
       ;Output:      BC is next entry number
       ;             HL points to variable data (just beyond detection string)
       ;             C set if problems
       ;             A is error code
       ;                    0: No more 8X* files found
       ;                    1: Not enough free memory (for the found level...subsequent calls may succeed)
       ;Take cluster number and entry number, get that 8X* variable in folder and copy to tempMem, return C if not possible, inc entry number
       ld (new_cluster),hl ;new_cluster used as temporary cluster number space
       ld (address),de ;address used as temporary detection string space
       call DOS_getNextFile
       jr c,msddNoMore
       ld (checksum),bc ;checksum used as temporary entry number space
       ;check extension and loop back if no match
       push hl
       pop ix
       ld hl,(new_cluster)
       ld a,(ix+8)
       cp '8'
       jr nz,msdDetect
       ld a,(ix+10)
       cp 'P'
       jr nz,msdDetect
       push ix
       pop hl
       ;open the file, check detection string, and loop back if no match
       ;DOS_openFileCont requirements:    C set if not found
       ;                                  HL is directory entry
       ;                                  DE is location to store file handle
       ld de,temp_handle
       push de
       call DOS_openFile_Cont
       pop bc
       ld de,0
       ld hl,37h
       call DOS_fileSeek
       ld hl,temp_handle
       ld de,packet_length
       ld bc,2
       call DOS_fileRead
       ld de,(packet_length)
       ld hl,39h
       add hl,de
       ld de,0
       ld bc,temp_handle
       push bc
       call DOS_fileSeek
       pop hl
       ld de,packet_length
       ld bc,2
       call DOS_fileRead
       ld hl,(packet_length)
       B_CALL EnoughMem
       ld a,1
       ret c
       ;check free space and return if not enough
       ;copy to tempMem and return
       ld hl,temp_handle
       ld de,(tempMem)
       push de
       ld bc,(packet_length)
       call DOS_fileRead
       ld de,(address)
       pop hl
       inc hl
       inc hl
msddLoop:
       ld a,(de)
       or a
       jr z,$F
       cp (hl)
       jr nz,msdDetect
       inc hl
       inc de
       jr msddLoop
$$:    ld bc,(checksum)
       ret
msddNoMore:
       xor a
       scf
       ret
